package ch.zhaw.gartenverwaltung.types;

import javafx.scene.image.Image;

import java.time.LocalDate;
import java.time.MonthDay;
import java.util.List;
import java.util.stream.Collectors;

import static java.time.temporal.ChronoUnit.DAYS;

public record Plant(
        long id,
        String name,
        String description,
        Image image,
        String spacing,
        int light,
        String soil,
        List<Pest> pests,
        List<GrowthPhase> lifecycle) {

    /**
     * remove all growthPhase which do not belong to the hardiness zone
     * @param zone hardiness zone
     */
    public void inZone(HardinessZone zone) {
        lifecycle.removeIf(growthPhase -> !growthPhase.zone().equals(zone));
    }

    /**
     * get all growthPhases of lifecycle group
     * @param group lifecycle group
     * @return list of growthPhases
     */
    public List<GrowthPhase> lifecycleForGroup(int group) {
        return lifecycle.stream()
                .filter(growthPhase -> growthPhase.group() == group)
                .collect(Collectors.toList());
    }

    /**
     * get sow date from given harvest day from lifecycle group
     * @param harvestDate date of the harvest
     * @param group lifecycle group
     * @return LocaleDate of sow date
     */
    public LocalDate sowDateFromHarvestDate(LocalDate harvestDate, int group) {
        return harvestDate.minusDays(timeToHarvest(group));
    }

    /**
     * calculate the days between sow and harvest day for lifecycle group
     * @param group the lifecycle group
     * @return Integer number of dates between sow and harvest day
     */
    public int timeToHarvest(int group) {
        List<GrowthPhase> activeLifecycle = lifecycleForGroup(group);
        GrowthPhase sow = activeLifecycle.stream()
                .filter(growthPhase -> !growthPhase.type().equals(GrowthPhaseType.SOW))
                .findFirst()
                .orElseThrow();
        GrowthPhase harvest = activeLifecycle.stream()
                .filter(growthPhase -> !growthPhase.type().equals(GrowthPhaseType.HARVEST))
                .findFirst()
                .orElseThrow();

        int currentYear = LocalDate.now().getYear();
        return (int) DAYS.between(harvest.startDate().atYear(currentYear), sow.startDate().atYear(currentYear));
    }


    public MonthDay getMaxDateForGrowthPhase(GrowthPhaseType growthPhase) {
        return lifecycle.stream()
                .filter(phase -> phase.type().equals(growthPhase))
                .findFirst()
                .map(GrowthPhase::endDate)
                .orElse(MonthDay.of(12, 31));
    }
    public MonthDay getMinDateForGrowthPhase(GrowthPhaseType growthPhase) {
        return lifecycle.stream()
                .filter(phase -> phase.type().equals(growthPhase))
                .findFirst()
                .map(GrowthPhase::startDate)
                .orElse(MonthDay.of(1, 1));
    }
}
